
class Disk:
    def __init__(self):
        print("Initializing disk...")
    def ioctl_get_sector_count(self):
        assert(0, "Not implemented.")
    def ioctl_get_sector_size(self):
        assert(0, "Not implemented.")
    def ioctl_get_block_size(self):
        assert(0, "Not implemented.")
    def ioctl_sync(self):
        pass
    def ioctl_trim(self):
        pass
    def read(self, sector, count) -> bytes:
        assert(0, "Not implemented.")
    def write(self, sector, count, buff: bytes):
        assert(0, "Not implemented.")


class RamDisk(Disk):
    def __init__(self, storage: bytearray, sector_size: int = 512, block_size: int = 1, sector_count = 0):
        self.storage = storage
        self.sector_size = sector_size
        self.block_size = block_size
        if (sector_count == 0):
            self.sector_count = len(storage) / sector_size
        else:
            self.sector_count = sector_count
        assert (len(storage) == self.sector_count * sector_size, "Cannot create ramdisk with len(storage) != sector_count * sector_size.")
    def ioctl_get_sector_count(self):
        return self.sector_count
    def ioctl_get_sector_size(self):
        return self.sector_size
    def ioctl_get_block_size(self):
        return self.block_size
    def ioctl_sync(self):
        pass
    def ioctl_trim(self):
        pass

    def read(self, sector, count) -> bytes:
        offset = sector * self.sector_size
        return self.storage[offset:offset+count]
    def write(self, sector, count, buff: bytes):
        assert(len(buff) == count, "Write failed. Non-matching write buffer.")
        offset = sector * self.sector_size
        self.storage[offset:offset+count] = buff




#!/usr/bin/python3

import fatfs
from fatfs import Partition, RamDisk

class NewPartition(Partition):
    """
    A helper class that handles partition mounts and initialization if necessary.
    """
    def __init__(self, disk = None):
        """
        Registers new partition on a disk (must have filesystem on it already),
        or creates a new disk image with empty filesystem.
        """
        if disk is not None:
            self.disk = disk
        else:
            self.disk = RamDisk(bytearray(512*256))
            Partition.__init__(self, self.disk)
            self.mkfs()

    def __enter__(self):
        self.mount()

    def __exit__(self, except_type, except_value, except_traceback):
        self.unmount()

    def storage(self):
        return self.disk.storage

    def dump(self, fname):
        with open(fname, "wb") as fh:
            fh.write(self.storage())



def test_fatfs_open():
    disk = RamDisk(bytearray(512*256))
    partition = Partition(disk)
    partition.mkfs()
    partition.mount()
    handle = partition.open("tf.txt", "wb")
    handle.close()
    partition.unmount()

def test_fatfs_write():
    partition = NewPartition()
    s = "HelloWorld!"
    written = 0
    with partition:
        with partition.open("/tf2.txt", "wb") as handle:
            written = handle.write(s)
    partition.dump("/tmp/test_fatfs_write.img")
    assert len(s) == written

def test_fatfs_mkdir():
    partition = NewPartition()
    with partition:
        partition.mkdir("testdir1")
        partition.mkdir("testdir1/a")
    partition.dump("/tmp/test_fatfs_mkdir.img")

def test_diskio():
    disk = RamDisk(bytearray(512*256))
    fatfs.wrapper.check_diskio(disk)



#!/usr/bin/python3 

import os
import sys
from setuptools import setup, find_packages, Extension
try:
    from Cython.Build import cythonize
except ImportError:
    cythonize = None


if 'sdist' in sys.argv and "CYTHONIZE" not in os.environ:
    raise Exception("Please use supplied makefile to build the package.")

# https://cython.readthedocs.io/en/latest/src/userguide/source_files_and_compilation.html#distributing-cython-modules
def no_cythonize(extensions, **_ignore):
    for extension in extensions:
        sources = []
        for sfile in extension.sources:
            path, ext = os.path.splitext(sfile)
            if ext in (".pyx", ".py"):
                if extension.language == "c++":
                    ext = ".cpp"
                else:
                    ext = ".c"
                sfile = path + ext
            sources.append(sfile)
        extension.sources[:] = sources
    return extensions


extensions = [
    Extension("wrapper", ["fatfs/wrapper.pyx", "fatfs/diskiocheck.c", "foreign/fatfs/source/ff.c", "foreign/fatfs/source/ffsystem.c", "foreign/fatfs/source/ffunicode.c"], include_dirs=["foreign/fatfs/source"]),
]

cythonize_env = bool(int(os.getenv("CYTHONIZE", 0)))

if cythonize_env and cythonize_env is None:
    raise Exception("Cython needs to be installed for CYTHONIZE=1")

CYTHONIZE = cythonize_env and cythonize is not None

if CYTHONIZE:
    compiler_directives = {"language_level": 3, "embedsignature": True}
    extensions = cythonize(extensions, compiler_directives=compiler_directives)
else:
    extensions = no_cythonize(extensions)

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='fatfs',
    version="0.1.2",
    author="Ladislav Laska",
    author_email="krakonos@krakonos.org",
    description="A wrapper around ChaN's FatFS library for FAT filesystem manipulation.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    ext_package='fatfs',
    ext_modules=extensions,
    url="https://github.com/krakonos/fatfs-python",
    #packages=['pyfatfs', 'pyfatfs.tests'],
    packages=find_packages(),
    setup_requires=['cython'],
    zip_safe=False,
    python_requires='>=3.8',
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development :: Embedded Systems",
        "Topic :: Software Development :: Libraries",
        "Topic :: System :: Filesystems",
    ],
)
